#!/usr/bin/perl
use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/modules";

use File::Copy;

# Finds the best map from amongst multiple maps generated from
# different random models. Uses the sum over the Fourier Shell
# Coefficient (FSC) as measure of quality.

# Input:
#   $listing_file - file containing names of auto3dem log files
#   $directory - data directory specified by setup_rmc

my ($logfile_list, $directory) = @ARGV;

# Generate list of logfiles to be parsed
my @logfiles = ();
open(LIST, $logfile_list);
while(<LIST>) {
    chomp;
    if(/\w/) {
	push(@logfiles,$_);
    }
}

# Parse logfiles and determine which one led to best model
my $best_fsc_sum =  0;
my $best_file;
foreach my $logfile (@logfiles) {
    unless(-r $logfile) {
	next;
    }
    my $fsc_sum = find_fsc_sum($logfile);
    if($fsc_sum > 0) {
	print "$logfile: $fsc_sum\n";
	if($fsc_sum > $best_fsc_sum) {
	    $best_fsc_sum = $fsc_sum;
	    $best_file = $logfile;
	}
    }
}

# Determine the last map generated by best random model
my $best_dir = "$directory/$best_file";
$best_dir =~ s/_log//;
my @maplist = <$best_dir/*_iter_*.pif>;
my $max_id = 0;
foreach my $map (@maplist) {
    my $id = $map;
    $id =~ s/.*_iter_//;
    $id =~ s/\.pif//;
    if($id > $max_id) {
	$max_id = $id;
    }
}
my $best_map = <$best_dir/*_iter_${max_id}.pif>;

# Move the best map into directory containing data files
copy "$best_map", "$directory/rmc.pif";

open(FB, ">RMC_bestmap_summary");
print FB "Best sum(FSC) = $best_fsc_sum (file = $best_file)\n";
print FB "best map = $best_map\n";
print FB "$best_map copied to $directory/rmc.pif\n";
print "Best sum(FSC) = $best_fsc_sum (file = $best_file)\n";
print "best map = $best_map\n";
print "$best_map copied to $directory/rmc.pif\n";
close(FB);

# ------------------------------------------------------

sub find_fsc_sum {

    # Extract FSC sum from the final iteration of auto3dem
    #
    # Input:
    #   $file - auto3dem log file
    #
    # Output:
    #   $fsc_sum - sum over FSC

    my ($file) = @_;
    open(FILE, $file);

    my $fsc_sum;

    while(<FILE>) {
	if(/Sum\s+over\s+FSC/) {
	    $fsc_sum = $_;
	}
    }

    if(defined $fsc_sum) {
	$fsc_sum =~ s/\s+$//; # Trim off trailing whitespace
	$fsc_sum =~ s/.*\s//; # Get rid of everthing except resolution
    } else {
	$fsc_sum = 0.0;       # Just return zero if data not found
    }
    
    close(FILE);
    return $fsc_sum;
}
